import numpy as np
from hdpca._d2_pca import D2PCA


class D2SquarePCA:

    def fit(self, X):
        X = np.asarray(X)
        mean = np.mean(X, 0)
        mean_subtracted = X - mean

        line_no = X.shape[0]
        height = X.shape[1]
        width = X.shape[2]
        row_covarience_matrix = np.zeros((height, height))
        column_covarience_matrix = np.zeros((width, width))

        for i in range(line_no):
            row_covarience_matrix += np.dot(mean_subtracted[i].T, mean_subtracted[i])
            column_covarience_matrix += np.dot(mean_subtracted[i], mean_subtracted[i].T)

        row_covarience_matrix /= line_no
        column_covarience_matrix /= line_no

        d_mat, p_mat = np.linalg.eig(row_covarience_matrix)
        if type(self.n_components) == float and 0.0 < self.n_components < 1.0:
            self.n_components = D2PCA.get_components_from_percentage(d_mat, self.n_components)
        self.bases_row = p_mat[:, 0:self.n_components]

        d_mat, p_mat = np.linalg.eig(column_covarience_matrix)
        if type(self.n_components_2) == float and 0.0 < self.n_components_2 < 1.0:
            self.n_components_2 = D2PCA.get_components_from_percentage(d_mat, self.n_components_2)
        self.bases_column = p_mat[:, 0:self.n_components_2]

    def transform(self, data):
        temp = np.dot(data, self.bases_row)
        line_no = data.shape[0]

        transform_data = np.zeros((line_no, self.n_components_2, self.n_components))
        for i in range(line_no):
            transform_data[i, :, :] = np.dot(self.bases_column.T, temp[i])
        return transform_data

    def __init__(self, n_components, n_components_2=None):
        if n_components_2 is None:
            self.n_components_2 = n_components
        self.n_components = n_components
        self.bases_row = None
        self.bases_column = None

    def inverse_transform(self, new_coordinates):
        return np.dot(self.bases_column, np.dot(new_coordinates, self.bases_row.T))



from __future__ import print_function
from setuptools import setup, find_packages

setup(
    name="hdpca",
    version="0.1.4",
    author="rileyge",  # 作者名字
    author_email="rlge@live.com",
    description="Hign Dimensional PCA. Including D2PCA and D2SquarePCA in 0.1.X versions.",
    license="Apache 2.0",
    url="https://gitee.com/rlge/hdpca",  # github地址或其他地址
    packages=find_packages(),
    include_package_data=True,
    # classifiers=[
    #     "Environment :: Web Environment",
    #     'Intended Audience :: Developers',
    #     'License :: Apache 2.0',
    #     'Operating System :: MacOS',
    #     'Operating System :: Microsoft',
    #     'Operating System :: Unix',
    #     'Topic :: Meachine Learning',
    #     'Topic :: Libraries :: Python Modules',
    #     'Programming Language :: Python :: 3.8'
    # ],
    install_requires=[
            'numpy>=1.14.0'   # 所需要包的版本号
    ],
    zip_safe=True,
)



import numpy as np


class D2PCA:
    @staticmethod
    def get_components_from_percentage(eigen_value, percentage):
        # sum of all eigen values
        eigen_sum = np.sum(eigen_value)
        sum_of_n_components = 0
        n_components = 0
        while sum_of_n_components < percentage * eigen_sum:
            sum_of_n_components += eigen_value[n_components]
            n_components += 1
        return n_components

    def transform(self, data):
        return np.dot(data, self.base)

    def __init__(self, n_components):
        self.n_components = n_components
        self.base = None

    def fit(self, X, y=None):
        X = np.asarray(X)
        mean = np.mean(X, 0)
        mean_subtracted = X - mean

        line_no = X.shape[0]
        height = X.shape[1]

        # creating emptymatrix for find covarience matrix
        covarience_matrix = np.zeros((height, height))
        for i in range(line_no):
            # multiplying net subtracted image with its transpose and adding in gt
            covarience_matrix += np.dot(mean_subtracted[i].T, mean_subtracted[i])
        covarience_matrix /= line_no

        # finding eigen values and eigen vectors
        d_mat, p_mat = np.linalg.eig(covarience_matrix)

        # finding first p important vectors
        if type(self.n_components) == float and 0.0 < self.n_components < 1.0:
            self.n_components = self.get_components_from_percentage(d_mat, self.n_components)
        self.base = p_mat[:, 0:self.n_components]

    def inverse_transform(self, new_coordinates):
        return np.dot(new_coordinates, self.base.T)
